import math

import numpy as np

from mower.solvers.fight.battle_choose.battle_filter import BattleFilterSolver
from mower.utils import config
from mower.utils.character_recognize import segment_team_select
from mower.utils.graph.utils import SceneGraphSolver
from mower.utils.image import cmatch, cropimg
from mower.utils.scene import Scene
from mower.utils.vector import va


# 按需填充干员
class BattleFillChooseSolver(SceneGraphSolver):
    def __init__(self) -> None:
        self.labels = [
            "获取时间",
            "信赖值",
            "生命上限",
            "攻击",
            "防御",
            "法术抗性",
            "再部署",
            "部署费用",
            "阻挡数",
            "攻击速度",
            "等级",
            "稀有度",
        ]

        self.profession = [
            "PIONEER",
            "WARRIOR",
            "TANK",
            "SNIPER",
            "CASTER",
            "MEDIC",
            "SUPPORT",
            "SPECIAL",
        ]

    def run(self, label: str, number: int) -> None:
        self.mode = None
        # 0是非职业的筛选，如信赖值或等级
        if label in self.labels:
            self.mode = 0
        # 1是干员职业的筛选
        elif label in self.profession:
            self.mode = 1
        else:
            raise KeyError(f"BattleAgentChooseSolver：填充干员标签错误{label}")
        self.label = label
        self.filter_done = False
        self.number = number
        super().run()
        pass

    def tag_choosed(self, profession: str = "ALL"):
        str = "choose_agent/profession/" + profession
        if pos := self.find(str):
            if arrow := self.find("choose_agent/profession/choose_arrow"):
                if (
                    math.sqrt(
                        math.pow(pos[0][0] - arrow[0][0], 2)
                        + math.pow(pos[0][1] - arrow[0][1], 2)
                    )
                    < 100
                ):
                    return False
                else:
                    self.tap(pos[0], interval=0.2)
                    return True
            else:
                self.tap(pos[0], interval=0.2)
                return True

    def filter(self):
        if pos := self.find("choose_agent/open_profession"):
            self.tap(pos, interval=0.1)
            return

        if self.mode == 1:
            return self.tag_choosed(self.label)
        elif self.mode == 0:
            if BattleFilterSolver().run(self.label):
                # 选全部干员
                if self.find("choose_agent/foldup"):
                    return True
                else:
                    return self.tag_choosed()
        return

    def fill_agent(self):
        segments = segment_team_select(config.recog.img)
        for i in segments:
            segment = [va(i[0], (0, -95)), va(i[1], (0, -190))]
            img = cropimg(config.recog.img, segment)
            template = np.zeros((5, 5, 3), dtype=np.uint8)
            template[:] = [120, 190, 220]
            if cmatch(img, template, thresh=50):
                continue
            self.tap(i, interval=0.1)

            config.recog.update()
            img = cropimg(config.recog.img, segment)
            if cmatch(img, template, thresh=50):
                self.number = self.number - 1
            else:
                return
            if self.number == 0:
                return True

        if self.number > 0:
            self.swipe_noinertia((1000, 540), (-1900, 0))
            return

    def transition(self) -> bool:
        if self.scene() == Scene.OPERATOR_AGENT_SELECT:
            if self.filter_done is False:
                if self.filter():
                    self.filter_done = True
                    return
            return self.fill_agent()
        else:
            return False
